#****************************************************************************
#**
#**  File     :  /cdimage/units/DEL0204/DEL0204_script.lua
#**  Author(s):  John Comes, David Tomandl, Jessica St. Croix, Matt Vainio
#**
#**  Summary  :  UEF Mongoose Gatling Bot with Advanced Jamming
#**
#**  Copyright  2008 4th Dimension
#****************************************************************************
#**
#**    This script snippet is required to be within the /lua/sim/Units.lua. This is borrowed from the CBFP. It
#**    adds a unit event that's fired when the unit is picked up by a transport or when it's dropped from one.
#**
#**    OnTransportAttach = function(self, attachBone, unit)
#**        oldUnit.OnTransportAttach(self, attachBone, unit)
#**        unit:OnAttachedToTransport(self)
#**    end,
#**
#**    OnAttachedToTransport = function(self, transport)
#**    end,
#**
#**    OnTransportDetach = function(self, attachBone, unit)
#**        oldUnit.OnTransportDetach(self, attachBone, unit)
#**        unit:OnDetachedToTransport(self)
#**    end,
#**
#**    OnDetachedToTransport = function(self, transport)
#**   end,   
#**
#****************************************************************************

local TWalkingLandUnit = import('/lua/terranunits.lua').TWalkingLandUnit
local TWeapons = import('/lua/terranweapons.lua')
local TDFPlasmaCannonWeapon = TWeapons.TDFPlasmaCannonWeapon
local TIFFragLauncherWeapon = TWeapons.TDFFragmentationGrenadeLauncherWeapon

local EffectUtils = import('/lua/effectutilities.lua')
local Effects = import('/lua/effecttemplates.lua')

DEL0204 = Class(TWalkingLandUnit) 
{
    Weapons = {
        GatlingCannon = Class(TDFPlasmaCannonWeapon) 
        {
            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                self.ExhaustEffects = EffectUtils.CreateBoneEffects( self.unit, 'Left_Arm_Barrel_Muzzle', self.unit:GetArmy(), Effects.WeaponSteam01 )
                TDFPlasmaCannonWeapon.PlayFxWeaponPackSequence(self)
            end,
        
            PlayFxRackSalvoChargeSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Left_Arm_Barrel', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(500)
                end
                TDFPlasmaCannonWeapon.PlayFxRackSalvoChargeSequence(self)
            end,            
            
            PlayFxRackSalvoReloadSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(200)
                end
                self.ExhaustEffects = EffectUtils.CreateBoneEffects( self.unit, 'Left_Arm_Barrel_Muzzle', self.unit:GetArmy(), Effects.WeaponSteam01 )
                TDFPlasmaCannonWeapon.PlayFxRackSalvoChargeSequence(self)
            end,
        },
        
        Grenade = Class(TIFFragLauncherWeapon) {}
    },

    OnStopBeingBuilt = function(self,builder,layer)
        TWalkingLandUnit.OnStopBeingBuilt(self,builder,layer)

        ###Sets up blip tables and starts jamming sequence
        self.BlipTable = {}
        self:SetJammingStatus(self:GetBlueprint().Intel.AltJamming.StartEnabled or true)
    end,

    OnScriptBitSet = function(self, bit)
        TWalkingLandUnit.OnScriptBitSet(self, bit)
        if not self:IsDead() then
            self:SetJammingStatus(false)
        end
    end,

    OnScriptBitClear = function(self, bit)
        TWalkingLandUnit.OnScriptBitClear(self, bit)
        if not self:IsDead() then
            self:SetJammingStatus(true)
        end
    end,

    OnAttachedToTransport = function(self, transport) 
        TWalkingLandUnit.OnAttachedToTransport(self, transport)
        self.PrevJammingStatus = self:GetJammingStatus()
        if self:GetBlueprint().Intel.AltJamming.DisabledInTransport then
            self:SetJammingStatus(false)
        end
    end,

    OnDetachedToTransport = function(self, transport) 
        TWalkingLandUnit.OnDetachedToTransport(self, transport)
        self:SetJammingStatus(self.PrevJammingStatus)
    end,

    SetJammingStatus = function(self, enabled)
        self.JammingActive = enabled
        if enabled then
            self:BlipSpawn()
            self:SetMaintenanceConsumptionActive()
        else
            self:SetMaintenanceConsumptionInactive()
            if self.BlipTable then 
            # Small bit of table manipulation to sort thru all of the avalible blips and remove them
                 if table.getn({self.BlipTable}) > 0 then
                    for k, v in self.BlipTable do 
                        IssueClearCommands({self.BlipTable[k]}) 
                        IssueDestroySelf({self.BlipTable[k]})
                    end 
                end
                self.BlipTable = {}
            end
        end
    end,

    GetJammingStatus = function(self)
        return self.JammingActive
    end,

    BlipSpawn = function(self)
        # put blip BPs here
        local BlipBPs = {
            air = {
                't1_airscout_blip','t1_fighter_blip','t1_bomber_blip','t2_gunship_blip','t2_fighter_bomber_blip',
            },
            ground = {
                't1_anti_air_blip','t1_artillery_blip','t1_bot_blip','t1_scout_blip','t1_tank_blip',
            },
            sea = {
                't1_frigate_blip','t2_sub_blip',
            },
            exp = {
                't4_land_blip',
            },
        }

        local spawnBlip
        local selectBP
        local parentPos = self:GetPosition()
        local bp = self:GetBlueprint().Intel.AltJamming
        local dist = bp.MaxDistance or Random(5, 20)
        local numAirBlips = bp.NumAirBlips or 0
        local numGroundBlips = bp.NumGroundBlips or 0
        local numSeaBlips = bp.NumSeaBlips or 0
        local numExpBlips = bp.NumExpBlips or 0
        local numTotalBlips = numAirBlips + numGroundBlips + numSeaBlips + numExpBlips

        local function CreateBlipUnit(selectBP)
            # creates radar blips
            return CreateUnitHPR(selectBP, self:GetArmy(), parentPos[1]+Random(-dist, dist), parentPos[2], parentPos[3]+ Random(-dist, dist), 0, 0, 0)
        end

        local function defineBlip(spawnBlip)
            # defines the blips made
            spawnBlip:SetCreator(self)
            spawnBlip:SetParent(self)
            spawnBlip:SetAllWeaponsEnabled(false)
            spawnBlip:SetDoNotTarget(true) 
            spawnBlip:SetUnSelectable(true)
            spawnBlip:SetVizToAllies('Never')
            spawnBlip:SetVizToFocusPlayer('Never')
            spawnBlip:SetVizToNeutrals('Never')
            spawnBlip:SetVizToEnemies('Never')
            table.insert(self.BlipTable, spawnBlip) 
            self.Trash:Add(spawnBlip)
            selectBP = nil
        end

        # Selects the blips to make
        if numTotalBlips >= 1 and not self:IsDead() then
            if numAirBlips >= 1 then
                for i = 0, (numAirBlips -1) do
                    selectBP = BlipBPs['air'][ Random( 1, table.getn( BlipBPs['air'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numGroundBlips >= 1 then
                for i = 0, (numGroundBlips -1) do
                    selectBP = BlipBPs['ground'][ Random( 1, table.getn( BlipBPs['ground'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numSeaBlips >= 1 then
                for i = 0, (numSeaBlips -1) do
                    selectBP = BlipBPs['sea'][ Random( 1, table.getn( BlipBPs['sea'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numExpBlips >= 1 then
                for i = 0, (numExpBlips -1) do
                    selectBP = BlipBPs['exp'][ Random( 1, table.getn( BlipBPs['exp'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
        end
    end,

    OnKilled = function(self, instigator, type, excessDamageRatio)
        self:SetJammingStatus(false)
        self.BlipTable = nil
        TWalkingLandUnit.OnKilled(self, instigator, type, excessDamageRatio)
    end,
}

TypeClass = DEL0204